
class Size:

    def __init__(self, id, name, ram, disk, bandwidth, price):
        self.id = id
        self.name = name
        self.ram = ram
        self.disk = disk
        self.bandwith = bandwidth
        self.price = price
